/***********************************************************************************************************************
**
** Copyright (C) 2021 BaseALT Ltd.
** Authors: Valery Sinelnikov <greh@basealt.ru>, Ivan Savin <svn17@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#include "alteratormodule.h"
/*!
 * \brief The _AlteratorModule struct Class fields
 */
struct _AlteratorModule {
    GObject parent_object;
    gchar* objectname;
    gchar* modulename;
    gchar* filename;
    GDBusNodeInfo *introspection_data;
    gchar* default_actionid;
    gchar* module_actionid;
    gint state;

};
G_DEFINE_TYPE (AlteratorModule, alterator_module, G_TYPE_OBJECT)

/*
 * Creates a GDBusNodeInfo* based on a file path for introspection,
 * storing it in an object variable
 */
static void
alterator_module_add_xml(AlteratorModule* self, const gchar *path)
{
    gchar *contents;
    GError *error = NULL;
    g_file_get_contents(path, &contents, NULL, NULL);
    self->introspection_data = g_dbus_node_info_new_for_xml(contents, &error);
    if(error) {
        self->state = FALSE;
        g_warning("alterator_module_add_xml() Object not created, bad xml\n %s", path);
        self->introspection_data = g_dbus_node_info_new_for_xml(ERROR_XML, NULL);
    }
    g_free(contents);
    g_info("alterator_module_add_xml()");
}
/*!
 * \brief alterator_module_state Returns the status of the object
 * \param self Pointer to the object
 * \return gboolean
 */
gboolean
alterator_module_state(AlteratorModule* self){
    g_info("alterator_module_state()");
    return self->state;
}

/*!
 * \brief alterator_module_get_introspection_data Returns the GDBusNodeInfo* stored in the object
 * \param self Pointer to the object
 * \return GDBusNodeInfo*
 */
GDBusNodeInfo*
alterator_module_get_introspection_data(AlteratorModule* self){
    g_info("alterator_module_get_introspection_data()");
    return self->introspection_data;
}

/*!
 * \brief alterator_module_set_objectname Changes objectname
 * \param self Pointer to the object
 * \param objectname New objectname
 */
void
alterator_module_set_objectname(AlteratorModule* self, gchar* objectname){
    self->objectname = objectname;
    g_info("alterator_module_set_objectname()");
}

/*!
 * \brief alterator_module_get_objectname Returns objectname
 * \param self Pointer to the object
 * \return objectname
 */
gchar*
alterator_module_get_objectname(AlteratorModule* self){
    GString* name = g_string_new("/ru/basealt/alterator");
    name = g_string_append(name, self->objectname);
    g_info("alterator_module_get_objectname()");
    return g_string_free(name, FALSE);
}
/*!
 * \brief alterator_module_get_modulename Returns modulename
 * \param self Pointer to the object
 * \return modulename
 */
gchar*
alterator_module_get_modulename(AlteratorModule* self){
    g_info("alterator_module_get_modulename()");
    return self->modulename;
}

static void
alterator_module_class_init(AlteratorModuleClass* self)
{
}

static void
alterator_module_init(AlteratorModule* self)
{
}

void
alterator_module_free(AlteratorModule* self){
    g_dbus_node_info_unref(self->introspection_data);
    g_free(self->objectname);
    g_free(self->modulename);
    g_free(self->filename);
    g_free(self->default_actionid);
    g_free(self->module_actionid);
}


static void
replace_char(gchar* text, gchar oldChar, gchar newChar)
{
    for(int i = 0; text[i] && (i < 100); i++){
        if(text[i] == oldChar){
            text[i] = newChar;
        }
    }
}

/*
 * Creates names for "woobus" and "dbus" from the filename
 */
static void
alterator_module_creating_name_fields(AlteratorModule* self)
{
    gchar *result = NULL;
    strtok(self->filename, ".");
    strtok(NULL, ".");
    strtok(NULL, ".");
    result = strtok(NULL, ".");
    g_message("module found: %s", result);
    gchar* modname = malloc(strlen(result) + 2);
    memset(modname, 0, strlen(result) + 2);
    strcat(modname, "/");
    strcat(modname, result);
    modname[strlen(result) + 1] = 0;
    self->modulename = modname;
    replace_char(result, '-', '_');
    gchar* name = malloc(strlen(result) + 2);
    memset(name, 0, strlen(result) + 2);
    strcat(name, "/");
    strcat(name, result);
    name[strlen(result) + 1] = 0;
    self->objectname = name;
    g_info("alterator_module_creating_name_fields()");

}

/*!
 * \brief alterator_module_new Creates a new AlteratorModule
 *  based on the passed introspection file path.
 * \param path Path to xml file
 * \param filename Specifies filename
 * \return New AlteratorModule
 */
AlteratorModule*
alterator_module_new(const gchar* path, gchar* filename) {
    AlteratorModule *self = ALTERATOR_MODULE(g_object_new(ALTERATOR_TYPE_MODULE, NULL));
    self->state = TRUE;
    self->filename = filename;
    GString *action = g_string_new(filename);
    g_string_erase(action, ((gint)action->len) - 4, 4);
    g_string_append(action, ".write");
    self->module_actionid = g_string_free(action, FALSE);
    replace_char(self->module_actionid, '-', '_');
    action = g_string_new_len(filename, 20);
    g_string_append(action, ".write");
    self->default_actionid = g_string_free(action, FALSE);
    alterator_module_creating_name_fields(self);
    alterator_module_add_xml(self, path);
    g_info("alterator_module_new()");
    return self;
}

gchar*
alterator_module_get_actionid_default(AlteratorModule* self){
    return self->default_actionid;
}

gchar*
alterator_module_get_actionid_module(AlteratorModule* self){
    return self->module_actionid;
}

